﻿
using UnityEngine;
using System.Collections;


public class PlayerControl : MonoBehaviour
{
    // movement config
    public float gravity = -15f;
    public float runSpeed = 8f;
    public float groundDamping = 20f; // how fast do we change direction? higher means faster
    public float inAirDamping = 5f;
    public float targetJumpHeight = 2f;
    public bool canDash = true;
    public float dashCoolDown = 0.25f;
    // public float jumpWaitTime = 2.0;

    [HideInInspector]
    public float rawMovementDirection = 1;
    //[HideInInspector]
    public float normalizedHorizontalSpeed = 0;

    CharacterController2D _controller;
    public RaycastHit2D lastControllerColliderHit;

    [HideInInspector]
    public Vector3 velocity;

    public int playerNumber = 1; //gibt an, ob es sich um player one, player two, etc. handelt. sollte nicht 0 sein;
    KeyCode goRightKeyCode, goLeftKeyCode, jumpKeyCode, dashKeyCode;

    void Start()
    {
        _controller = GetComponent<CharacterController2D>();
        _controller.onControllerCollidedEvent += onControllerCollider;

        switch (playerNumber)
        {
            case 1:
               setKeyCodes(KeyCode.D, KeyCode.A, KeyCode.W, KeyCode.S);
                break;
            case 2:
                setKeyCodes(KeyCode.RightArrow, KeyCode.LeftArrow, KeyCode.UpArrow, KeyCode.DownArrow);
                break;
                //TODO player 3 and 4
            default:
                break;
        }
    }

    private void setKeyCodes(KeyCode right, KeyCode left, KeyCode up, KeyCode dash)
    {
        goRightKeyCode = right;
        goLeftKeyCode = left;
        jumpKeyCode = up;
        dashKeyCode = dash;
    }


    void onControllerCollider(RaycastHit2D hit)
    {
        // bail out on plain old ground hits
        if (hit.normal.y == 1f)
            return;

        // logs any collider hits
        //Debug.Log( "flags: " + _controller.collisionState + ", hit.normal: " + hit.normal );
    }


    void Update()
    {
        // grab our current velocity to use as a base for all calculations
        velocity = _controller.velocity;

        if (_controller.isGrounded)
            velocity.y = 0;

        
        if (Input.GetKey(goRightKeyCode))
        {
            normalizedHorizontalSpeed = 1;
            if (transform.localScale.x < 0f)
                transform.localScale = new Vector3(-transform.localScale.x, transform.localScale.y, transform.localScale.z);

            //if (_controller.isGrounded)
        }
        else if (Input.GetKey(goLeftKeyCode))
        {
            normalizedHorizontalSpeed = -1;
            if (transform.localScale.x > 0f)
                transform.localScale = new Vector3(-transform.localScale.x, transform.localScale.y, transform.localScale.z);

            //if (_controller.isGrounded)
        }
        else
        {
            normalizedHorizontalSpeed = 0;

            //if (_controller.isGrounded)
        }

        if (canDash && Input.GetKey (dashKeyCode))
        {
            float direction = Mathf.Sign (transform.localScale.x);
            if (normalizedHorizontalSpeed != 0)
            {
                // player pressed a movement button so prefer the new direction
                direction = Mathf.Sign (normalizedHorizontalSpeed);
            }

            normalizedHorizontalSpeed = 5 * direction;
            canDash = false;
            Invoke("EnableDash", dashCoolDown);
        }

        if (Input.GetKeyDown(jumpKeyCode))
        {
            //to avoid DOUBLE JUMP
            if (_controller.isGrounded)
            {
                velocity.y = Mathf.Sqrt(2f * targetJumpHeight * -gravity);
            }
        }


        // apply horizontal speed smoothing it
        var smoothedMovementFactor = _controller.isGrounded ? groundDamping : inAirDamping; // how fast do we change direction?
        velocity.x = Mathf.Lerp(velocity.x, normalizedHorizontalSpeed * rawMovementDirection * runSpeed, Time.deltaTime * smoothedMovementFactor);

            // apply gravity before moving
            velocity.y += gravity * Time.deltaTime;

        _controller.move(velocity * Time.deltaTime);

    }

    void EnableDash()
    {
        canDash = true;
    }
}
